#!/usr/bin/env python
import sys
import logging
import argparse
from matador.commands import commands


def _setup_logging(logging_destination='console', verbosity='INFO'):
    logHandlers = {
        'console': logging.StreamHandler(),
        'none': logging.NullHandler(),
        # 'file': logging.FileHandler('./matador.log')
    }
    logHandler = logHandlers[logging_destination]

    logFormatters = {
        'console': '%(message)s',
        'none': '',
        'file': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    }
    logFormatter = logging.Formatter(logFormatters[logging_destination])

    logHandler.setFormatter(logFormatter)
    logger = logging.getLogger('matador')
    level = logging.getLevelName(verbosity.upper())
    logger.setLevel(level)
    logger.addHandler(logHandler)


def execute_command():
    """Entry point for command line executable."""

    parser = argparse.ArgumentParser(
        description="Taming the bull: Change management for Agresso systems")

    parser.add_argument(
        'command',
        type=str,
        help='Command')

    parser.add_argument(
        '-l', '--logging',
        type=str,
        default='console',
        dest='logging_destination',
        help='logging (none, console or file)')

    parser.add_argument(
        '-v', '--verbosity',
        type=str,
        default='INFO',
        help='Logging level. DEBUG, INFO, ERROR or CRITICAL')

    try:
        args, sub_args = parser.parse_known_args()
        _setup_logging(args.logging_destination, args.verbosity)
    except:
        parser.print_help()
        sys.exit()

    commands[args.command]()



#!/usr/bin/env python
import logging
import argparse
from matador.session import Session


class Command(object):

    def __init__(self, **kwargs):
        if kwargs:
            # If kwargs have been supplied, use these in same way argsparse
            # would.
            # Mainly used for testing, but could be used to run commands from
            # within python scripts.

            # We can't add attributes to instances of Object, so we need create
            # a new class.
            class Args(object):
                pass

            self.args = Args()
            for key, value in kwargs.items():
                setattr(self.args, key, value)
        else:
            # If the command is created from the command line, we'll have
            # arguments to parse.
            parser = argparse.ArgumentParser(
                description="Taming the bull: Change management for Agresso systems")
            self._add_arguments(parser)
            self.args, unknown = parser.parse_known_args()

        self._logger = logging.getLogger(__name__)
        Session.initialise()
        self._execute()

    def _add_arguments(self, parser):
        pass

    def _execute(self):
        raise NotImplementedError



#!/usr/bin/env python
from pathlib import Path
import os
import subprocess
from string import Template
from .command import Command
from matador.session import Session


def _connection_string(dbms, connection, user, password):
    if dbms.lower() == 'oracle':
        return user + '/' + password + '@' + connection


def _sql_script(file_path):
    with file_path.open('r') as f:
        script = f.read()
        f.close()
    return script


def run_sql_script(logger, file_path):
    file = Path(file_path)
    message = Template(
        'Matador: Executing ${file} against ${connection} \n')
    substitutions = {
        'file': file.name,
        'connection': Session.environment['connection']
    }
    logger.info(message.substitute(substitutions))

    script = _sql_script(file)
    connection_string = _connection_string(
        Session.environment['dbms'],
        Session.environment['connection'],
        Session.credentials['user'],
        Session.credentials['password'])

    os.chdir(str(file.parent))

    if Session.environment['dbms'].lower() == 'oracle':
        script += '\nshow error'
        process = subprocess.Popen(
            ['sqlplus', '-S', '-L', connection_string],
            stdin=subprocess.PIPE,
            stderr=subprocess.PIPE)
        process.stdin.write(script.encode('utf-8'))
        process.stdin.close()
        process.wait()


class RunSqlScript(Command):

    def _add_arguments(self, parser):

        parser.add_argument(
            '-d', '--directory',
            type=str,
            required=True,
            help='Directory containing script')

        parser.add_argument(
            '-f', '--file',
            type=str,
            required=True,
            help='Script file name')

        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment')

    def _execute(self):
        Session.set_environment(self.args.environment)

        file_path = os.path.join(self.args.directory, self.args.file)

        run_sql_script(
            self._logger,
            file_path)



from .command import Command
from matador.session import Session
import win32serviceutil as win32


class ActionService(Command):

    def _is_running(self, label, name, with_logging=False):
        if win32.QueryServiceStatus(name, machine=self.server)[1] == 4:
            message = ('%s - Running on %s' % (label, self.args.environment))
            running = True
        else:
            message = (
                '%s - Not Running on %s' % (label, self.args.environment))
            running = False

        if with_logging:
            self._logger.info(message)

        return running

    def _add_arguments(self, parser):
        parser.prog = 'matador start/stop-services'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-s', '--service',
            type=str,
            default='all',
            help='Agresso service label')

    def _execute(self):
        self.server = Session.environments[self.args.environment]['abwServer']
        self.services = Session.environments[self.args.environment]['services']


class StartService(ActionService):

    def _start(self, label, name):
        if not self._is_running(label, name):
            win32.StartService(name, machine=self.server)
            self._is_running(
                self.args.service,
                self.services[self.args.service],
                True)
        else:
            self._logger.info(
                'Cannot start %s on %s - It is already running' %
                (label, self.args.environment))

    def _execute(self):
        super(StartService, self)._execute()
        if self.args.service == 'all':
            for label, name in self.services.items():
                self._start(label, name)
        else:
            self._start(self.args.service, self.services[self.args.service])


class StopService(ActionService):

    def _stop(self, label, name):
        if self._is_running(label, name):
            win32.StopService(name, machine=self.server)
            self._is_running(
                self.args.service,
                self.services[self.args.service],
                True)
        else:
            self._logger.info(
                'Cannot stop %s on %s - It is not currently running' %
                (label, self.args.environment))

    def _execute(self):
        super(StopService, self)._execute()
        if self.args.service == 'all':
            for label, name in self.services.items():
                self._stop(label, name)
        else:
            self._stop(self.args.service, self.services[self.args.service])


class RestartService(ActionService):

    def _restart(self, label, name):
        if self._is_running(label, name):
            win32.RestartService(name, machine=self.server)
            self._is_running(
                self.args.service,
                self.services[self.args.service],
                True)
        else:
            self._logger.info(
                'Cannot restart %s on %s - It is not currently running' %
                (label, self.args.environment))

    def _execute(self):
        super(RestartService, self)._execute()
        if self.args.service == 'all':
            for label, name in self.services.items():
                self._restart(label, name)
        else:
            self._restart(self.args.service, self.services[self.args.service])


class ServiceStatus(ActionService):

    def _execute(self):
        super(ServiceStatus, self)._execute()
        if self.args.service == 'all':
            for label, name in self.services.items():
                self._is_running(label, name, True)
        else:
            self._is_running(
                self.args.service,
                self.services[self.args.service],
                True)



#!/usr/bin/env python
from pathlib import Path
from matador.session import Session
from .deployment import DeploymentCommand
from matador.commands.run_sql_script import run_sql_script
from matador import git


def _fetch_script(repo, script_path, commit_ref, target_folder):
    script = Path(repo.path, script_path)
    target_script = Path(target_folder, script.name)

    git.checkout(repo, commit_ref)

    with script.open('r') as f:
        original_text = f.read()
        f.close()

    new_text = git.substitute_keywords(original_text, repo, commit_ref)

    with target_script.open('w') as f:
        f.write(new_text)
        f.close()

    return target_script


class DeploySqlScript(DeploymentCommand):

    def _execute(self):
        path = Path(self.args[0])

        if str(path.parent) == '.':
            script = Path(Session.deployment_folder, path)
        else:
            commit = self.args[1]
            script = _fetch_script(
                Session.matador_repo, path, commit, Session.deployment_folder)

        run_sql_script(self._logger, str(script))


class DeployOraclePackage(DeploymentCommand):

    def _execute(self):
        package_name = self.args[0]
        commit = self.args[1]

        repo_folder = Session.matador_repository_folder
        package_folder = Path(
            repo_folder, 'src', 'db_objects', 'packages', package_name)
        package_spec = Path(package_folder, package_name + '.pks')
        package_body = Path(package_folder, package_name + '.pkb')

        spec_script = _fetch_script(
            Session.matador_repo, package_spec, commit,
            Session.deployment_folder)
        body_script = _fetch_script(
            Session.matador_repo, package_body, commit,
            Session.deployment_folder)

        run_sql_script(self._logger, spec_script)
        run_sql_script(self._logger, body_script)



import logging
from dulwich.client import LocalGitClient
from dulwich.index import build_index_from_tree
from dulwich.objects import format_timezone
from time import strftime, gmtime
import re

logger = logging.getLogger(__name__)


def stage_file(repo, file):
    """Stage a file to the index

    The equivalent of :code:`git add`

    Parameters
    ----------
    repo : dulwich.repo.Repo
    file : pathlib.Path
    """
    file_path = str(file.relative_to(repo.path))
    repo.stage(file_path)


def commit(repo, message):
    """Commit the index to the repository

    The equivalent of :code:`git commit`

    Parameters
    ----------
    repo : dulwich.repo.Repo
    message : str
    """
    message = bytes(message, encoding='UTF-8')
    repo.do_commit(message)


def fetch_all(source_repo, target_repo, remote_name=None):
    """Fetch branches and tags from a remote repository

    Unlike :code:`git fetch`, this will also update local branches to point at the
    same commit as their remote counterpart.

     Parameters
    ----------
    source_repo : dulwich.repo.Repo
    target_repo : dulwich.repo.Repo
    remote_name : str
    """
    if remote_name is None:
        remote_name = 'origin'

    refs = LocalGitClient().fetch(source_repo.path, target_repo)

    for key, value in refs.items():
        remote_key = key.replace(
            b'heads', b'remotes/%s' % bytes(remote_name, encoding='UTF-8'))
        target_repo.refs[remote_key] = value
        target_repo[key] = value


def full_ref(repo, ref):
    """Generate a fully qualified git reference

    If ref refers to a branch or tag, the function will return the correct
    fully qualified reference, otherwise it returns the ref as provided.

    e.g. for ref 'master', return 'refs/heads/master'

    Parameters
    ----------
    repo : dulwich.repo.Repo

    Returns
    -------
    str
    """
    refs = repo.refs.keys()
    for ref_type in ['refs/heads/', 'refs/tags/']:
        full_ref = ref_type + ref
        if bytes(full_ref, encoding='ascii') in refs:
            ref = full_ref
    return ref


def checkout(repo, ref=None):
    """Checkout the commit from a given ref to the working directory

    The equivalent of :code:`git checkout`

    Parameters
    ----------
    repo : dulwich.repo.Repo
    ref : str

    Returns
    -------
    list
    """
    if ref is None:
        ref = repo.head()
    else:
        ref = bytes(full_ref(repo, ref), encoding='ascii')
    index = repo.index_path()
    tree_id = repo[ref].tree
    build_index_from_tree(repo.path, index, repo.object_store, tree_id)
    return [repo.object_store.iter_tree_contents(tree_id)]


def substitute_keywords(text, repo, ref):
    """Perform keyword substitution on given text

    Substitutes the keywords 'version:', 'date:' and 'author:' with the
    relevant attributes extracted from the repository for the given ref.

    Parameters
    ----------
    text : str
    repo : dulwich.repo.Repo
    ref : str

    Returns
    -------
    str
    """
    new_text = ''
    expanded_ref = full_ref(repo, ref)

    try:
        sha = repo.refs[bytes(expanded_ref, encoding='ascii')]
        short_sha = sha[:7].decode(encoding='ascii')

        commit = repo.get_object(sha)
        commit_time = strftime('%Y-%m-%d %H:%M:%S', gmtime(commit.commit_time))
        timezone = format_timezone(
            commit.commit_timezone).decode(encoding='ascii')
        commit_timestamp = commit_time + ' ' + timezone
        author = commit.author.decode(encoding='ascii')

        if ref.startswith('refs/tags'):
            version = 'Tag %s (%s)' % (ref, short_sha)
        else:
            version = short_sha

        substitutions = {
            'version': version,
            'date': commit_timestamp,
            'author': author
        }

        for line in text.splitlines(keepends=True):
            for key, value in substitutions.items():
                rexp = '%s:.*' % key
                line = re.sub(rexp, '%s: %s' % (key, value), line)
            new_text += line

    except KeyError:
        logger.error('%s is not a valid branch or tag' % ref)

    return new_text



#!/usr/bin/env python
from .command import Command
from .deploy_ticket import execute_ticket
from matador.session import Session
from matador import git
from pathlib import Path
import shutil
import yaml
from importlib.machinery import SourceFileLoader


class ActionPackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador deploy-package'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Package name')

        parser.add_argument(
            '-c', '--commit',
            type=str,
            default='none',
            help='Commit or tag ID')

    @staticmethod
    def _checkout_package(package, commit):
        repo_folder = Session.matador_repository_folder
        package_folder = Path(
            Session.matador_packages_folder, package)

        shutil.rmtree(str(package_folder), ignore_errors=True)

        Session.update_repository()

        git.checkout(Session.matador_repo, commit)

        src = Path(repo_folder, 'deploy', 'packages', package)
        shutil.copytree(str(src), str(package_folder))

    def _execute(self):
        Session.set_environment(self.args.environment)
        if self.args.commit == 'none':
            commit = None
        else:
            commit = self.args.commit
        self._checkout_package(self.args.package, commit)


class DeployPackage(ActionPackage):

    def _execute(self):
        super(DeployPackage, self)._execute()
        if self.args.commit == 'none':
            commit = None
        else:
            commit = self.args.commit
        package_folder = Path(
            Session.matador_packages_folder, self.args.package)
        Session.deployment_folder = package_folder
        ticketsFile = Path(package_folder, 'tickets.yml')

        file = ticketsFile.open('r')
        tickets = yaml.load(file)
        for ticket in tickets:
            self._logger.info('*' * 25)
            self._logger.info('Deploying ticket %s' % ticket)
            self._logger.info('*' * 25)
            execute_ticket(str(ticket), 'deploy', commit, True)


class RemovePackage(ActionPackage):

    def _execute(self):
        super(RemovePackage, self)._execute()
        package_folder = Path(
            Session.matador_packages_folder, self.args.package)
        Session.deployment_folder = package_folder
        sourceFile = Path(package_folder, 'remove.py')

        SourceFileLoader('remove', str(sourceFile)).load_module()



#!/usr/bin/env python
from .command import Command
from matador.commands.deployment import *
from matador.session import Session
from matador import git
from pathlib import Path
import shutil
from importlib.machinery import SourceFileLoader


def _checkout_ticket(ticket, repo, ticket_folder, commit):
    git.checkout(repo, commit)
    src = Path(repo.path, 'deploy', 'tickets', ticket)
    shutil.rmtree(str(ticket_folder), ignore_errors=True)
    shutil.copytree(str(src), str(ticket_folder))


def execute_ticket(ticket, action, commit, packaged=False):
    ticket_folder = Path(Session.matador_tickets_folder, ticket)
    Session.deployment_folder = ticket_folder

    shutil.rmtree(str(ticket_folder), ignore_errors=True)

    if not packaged:
        Session.update_repository()

    _checkout_ticket(ticket, Session.matador_repo, ticket_folder, commit)

    actionFile = Path(action + '.py')
    sourceFile = Path(ticket_folder, actionFile)
    SourceFileLoader(action, str(sourceFile)).load_module()


class ActionTicket(Command):
    action = 'None'

    def _add_arguments(self, parser):
        parser.prog = 'matador deploy-ticket'
        parser.add_argument(
            '-e', '--environment',
            type=str,
            required=True,
            help='Agresso environment name')

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

        parser.add_argument(
            '-c', '--commit',
            type=str,
            default='none',
            help='Commit or tag ID')

        parser.add_argument(
            '-p', '--packaged',
            type=bool,
            default=False,
            help='Whether this deployment is part of a package')

    def _execute(self):
        Session.set_environment(self.args.environment)
        if self.args.commit == 'none':
            commit = None
        else:
            commit = self.args.commit
        execute_ticket(self.args.ticket, self.action, commit, False)


class DeployTicket(ActionTicket):
    action = 'deploy'


class RemoveTicket(ActionTicket):
    action = 'remove'



#!/usr/bin/env python
import logging
import yaml
from dulwich.repo import Repo
from dulwich.errors import NotGitRepository
from configparser import ConfigParser
from pathlib import Path
from matador import git

logger = logging.getLogger(__name__)


def get_environments(project_folder):
    """Fetch environment details from their config file."""
    file_path = Path(
        project_folder, 'config', 'environments.yml')
    try:
        file = file_path.open('r')
        environments = yaml.load(file)
        if environments:
            return environments
        else:
            raise ValueError()
    except FileNotFoundError:
        logger.error('Cannot find environments.yml file')
    except ValueError:
        logger.error('environments.yml exists but is empty')


def get_credentials(project_folder):
    """Fetch credential details from their config file."""
    file_path = Path(
        project_folder, 'config', 'credentials.yml')
    try:
        file = file_path.open('r')
        credentials = yaml.load(file)
        if credentials:
            return credentials
        else:
            raise ValueError()
    except FileNotFoundError:
        logger.error('Cannot find credentials.yml file')
    except ValueError:
        logger.error('credentials.yml exists but is empty')


def initialise_repository(proj_folder, repo_folder):
    """Create a git repository for matador to use."""
    config_file = Path(repo_folder, '.git', 'config')
    config = ConfigParser()

    repo = Repo.init(str(repo_folder))
    config.read(str(config_file))

    config['core']['sparsecheckout'] = 'true'
    config['remote "origin"'] = {
        'url': proj_folder,
        'fetch': '+refs/heads/*:refs/remotes/origin/*'
    }

    with config_file.open('w') as f:
        config.write(f)
        f.close()

    sparse_checkout_file = Path(
        repo_folder, '.git', 'info', 'sparse-checkout')
    with sparse_checkout_file.open('a') as f:
        f.write('/src\n')
        f.write('/deploy\n')
        f.close()

    return repo


class Session(object):

    """A class to hold variables for a matador session."""

    project_repo = None
    matador_repo = None
    environment = None

    @classmethod
    def initialise(self):
        if self.project_repo is None:
            self.project_repo = Repo.discover()
            self.project_folder = Path(self.project_repo.path)

            self.project = self.project_folder.name

            self.matador_project_folder = Path(
                Path.home(), '.matador', self.project)

            self.matador_repository_folder = Path(
                self.matador_project_folder, 'repository')

            self.environments = get_environments(self.project_folder)
        else:
            return

    @classmethod
    def _initialise_matador_repository(self):
        """Initialise a git repository for matador to use."""
        Path.mkdir(
            self.matador_project_folder, parents=True, exist_ok=True)
        Path.mkdir(
            self.matador_repository_folder, parents=True, exist_ok=True)

        try:
            repo = Repo(str(self.matador_repository_folder))
        except NotGitRepository:
            repo = initialise_repository(
                self.project_folder, self.matador_repository_folder)

        self.matador_repo = repo

    @classmethod
    def set_environment(self, environment):
        """Set a specific environment for those commands which require it."""

        if self.project_repo is None:
            self.initialise()

        if self.environment is None:
            self._initialise_matador_repository()
            self.environment = self.environments[environment]
            credentials = get_credentials(self.project_folder)
            self.credentials = credentials[environment]

            self.matador_environment_folder = Path(
                self.matador_project_folder, environment)
            self.matador_tickets_folder = Path(
                self.matador_environment_folder, 'tickets')
            self.matador_packages_folder = Path(
                self.matador_environment_folder, 'packages')

            Path.mkdir(
                self.matador_environment_folder, parents=True, exist_ok=True)
            Path.mkdir(
                self.matador_tickets_folder, parents=True, exist_ok=True)
            Path.mkdir(
                self.matador_packages_folder, parents=True, exist_ok=True)
        else:
            return

    @classmethod
    def update_repository(self):
        """Fetch all from the project repo to the matador repo."""

        if self.project_repo is None:
            self.initialise()

        if self.matador_repo is None:
            self._initialise_matador_repository()

        git.fetch_all(self.project_repo, self.matador_repo)

    @classmethod
    def clear(self):
        self.project_repo = None
        self.matador_repo = None
        self.environment = None



import logging


class DeploymentCommand(object):

    def __init__(self, *args):
        self._logger = logging.getLogger(__name__)
        self.args = args
        self._execute()

    def _execute(self):
        raise NotImplementedError



from .run_sql_script import RunSqlScript
from .deploy_ticket import DeployTicket, RemoveTicket
from .deploy_package import DeployPackage, RemovePackage
from .create import CreateTicket, CreatePackage, AddTicketToPackage
import os

commands = {
    'run-sql-script': RunSqlScript,
    'deploy-ticket': DeployTicket,
    'remove-ticket': RemoveTicket,
    'deploy-package': DeployPackage,
    'remove-package': RemovePackage,
    'create-ticket': CreateTicket,
    'create-package': CreatePackage,
    'add-t2p': AddTicketToPackage,
}

if os.name == 'nt':
    from .abw_service import (
        StartService, StopService, RestartService, ServiceStatus)

    commands['start-service'] = StartService
    commands['stop-service'] = StopService
    commands['restart-service'] = RestartService
    commands['service-status'] = ServiceStatus



from setuptools import setup, find_packages

# Work around mbcs bug in distutils.
# http://bugs.python.org/issue10945
import codecs
try:
    codecs.lookup('mbcs')
except LookupError:
    ascii = codecs.lookup('ascii')
    func = lambda name, enc=ascii: {True: enc}.get(name=='mbcs')
    codecs.register(func)

setup(
    name='Matador',
    version='1.5.1',
    author='Owen Campbell',
    author_email='owen.campbell@empiria.co.uk',
    entry_points={
        'console_scripts': [
            'matador = matador.management:execute_command',
        ],
    },
    options={
        'build_scripts': {
            'executable': '/usr/bin/env python3',
        },
    },
    url='http://www.empiria.co.uk',
    packages=find_packages(),
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    install_requires=['pyyaml', 'dulwich'],
    license='The MIT License (MIT)',
    description='Change management for Agresso systems',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English'
        ]
    )



from .command import Command
from matador.session import Session
from matador import git
from pathlib import Path


class CreateTicket(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador create-ticket'

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        ticket_folder = Path(
            Session.project_folder, 'deploy', 'tickets', self.args.ticket)
        Path.mkdir(ticket_folder, parents=True, exist_ok=True)
        deploy_file = Path(ticket_folder, 'deploy.py')

        with deploy_file.open('w') as f:
            f.write('from matador.commands.deployment import *\n\n')
            f.close()

        git.stage_file(Session.project_repo, deploy_file)
        git.commit(Session.project_repo, 'Create ticket %s' % self.args.ticket)


class CreatePackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador create-package'

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        package_folder = Path(
            Session.project_folder, 'deploy', 'packages', self.args.package)
        Path.mkdir(package_folder, parents=True, exist_ok=True)

        package_file = Path(package_folder, 'tickets.yml')
        with package_file.open('w') as f:
            f.write(
                '# List each ticket on a separate line preceded by - . e.g.\n')
            f.write('# - 30\n')
            f.write('# - 31\n')
            f.close()
        git.stage_file(Session.project_repo, package_file)

        remove_file = Path(package_folder, 'remove.py')
        with remove_file.open('w') as f:
            f.write('from matador.commands.deployment import *\n\n')
            f.close()
        git.stage_file(Session.project_repo, remove_file)

        git.commit(
            Session.project_repo, 'Create package %s' % self.args.package)


class AddTicketToPackage(Command):

    def _add_arguments(self, parser):
        parser.prog = 'matador add-t2p'

        parser.add_argument(
            '-t', '--ticket',
            type=str,
            required=True,
            help='Ticket name')

        parser.add_argument(
            '-p', '--package',
            type=str,
            required=True,
            help='Ticket name')

    def _execute(self):
        package_file = Path(
            Session.project_folder, 'deploy', 'packages', self.args.package,
            'tickets.yml')

        with package_file.open('a') as f:
            f.write('- %s\n' % self.args.ticket)
            f.close()

        git.stage_file(Session.project_repo, package_file)
        git.commit(Session.project_repo, 'Add ticket %s to package %s' % (
                self.args.ticket, self.args.package))
